package com.example.cells.IO.Handlers;

import java.util.ArrayList;

import com.example.cells.IO.ObjectFactory;
import com.example.cells.IO.Pool;
import com.example.cells.IO.Input.TouchEvent;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;

public class TouchHandler implements OnTouchListener{
	private boolean[] _isTouched; //array of pointers
	private Pool<TouchEvent> _touchEventPool;
	private ObjectFactory _factory;
	private ArrayList<TouchEvent> _touchEventsBufferOne = new ArrayList<TouchEvent>();
	private ArrayList<TouchEvent> _touchEventsBufferTwo = new ArrayList<TouchEvent>();
	
	//no access modifiers for speed; accessible to package
	float _scaleX, _scaleY;
	int[] _touchX, _touchY;
	//logcat debugging
	private final String TAG = "MultiTouch Handler: ";
	
	public TouchHandler(View view, float scaleX, float scaleY){
		//create the specific factory for the pool
		_factory = new ObjectFactory<TouchEvent>(){
			public TouchEvent createObject(){
				return new TouchEvent();
			}
			//Log.v(TAG, "---From the TouchEvent createObject() in the TouchHandler constructor---");
		};//ObjectFactory
		
		_touchEventPool = new Pool<TouchEvent>(_factory, 100);
		
		//register this as the listener for the view
		view.setOnTouchListener(this);
		
		this._scaleX = scaleX;
		this._scaleY = scaleY;
		this._isTouched = new boolean[20];
		this._touchX = new int[20];
		this._touchY = new int[20];
		
	}//constructor
	
	//implements OnTouchListener
	public boolean onTouch(View view, MotionEvent event){
		//return true tells android the MotionEvent object has been consumed and doesnt need to be passed elsewhere
		//tells android to keep sending touch sequences to this method rather than elsewhere p599 
		boolean bool = true;
		
		//UI Thread as the view is passed into separate thread in CWSurfaceView
		//get events from the listener attached to the view, and store in the Input.TouchEvent object
		synchronized(this){
			//major debugging
			Log.v(TAG, "----from TouchHandler onTouch() method ----");
			describeMotionEventsforDebug(view, event);
			
			int action = event.getActionMasked();
			int pointerIndex = event.getActionIndex();
			int pointerId = event.getPointerId(pointerIndex);
			TouchEvent myTouchEvent; //will be gotten from the pool-factory construct later on
			
			switch(action){
			case MotionEvent.ACTION_DOWN:
			case MotionEvent.ACTION_POINTER_DOWN: //non-primary pointer has gone down
				myTouchEvent = _touchEventPool.getObject();
				myTouchEvent.type = myTouchEvent.TOUCH_DOWN;
				myTouchEvent.pointer = pointerId;
				this._isTouched[pointerId] =true;
				_touchEventsBufferOne.add(myTouchEvent);
				break;
			case MotionEvent.ACTION_UP:
			case MotionEvent.ACTION_POINTER_UP: //non-primary pointer has gone up
			case MotionEvent.ACTION_CANCEL: //probably don't need this but is more robust
				myTouchEvent = _touchEventPool.getObject();
				myTouchEvent.type = myTouchEvent.TOUCH_UP;
				myTouchEvent.pointer = pointerId;
				this._isTouched[pointerId] =false;
				_touchEventsBufferOne.add(myTouchEvent);
				break;
			case MotionEvent.ACTION_MOVE:
				//will change when I get the game dynamics more fine-tuned
				int numOfPointers = event.getPointerCount(); //how many fingers on screen
				int ptrIndex = 0; //index always 0
				while(ptrIndex < numOfPointers){
					myTouchEvent = _touchEventPool.getObject();
					myTouchEvent.pointer = event.getPointerId(ptrIndex);
					myTouchEvent.type = myTouchEvent.TOUCH_DRAGGED;
					_touchEventsBufferOne.add(myTouchEvent);
				}
				break;
			}//switch
		
		return bool;
	}//synchronized
	}//onTouch	
	
	//this is for debugging the multi-touch motion events. will be commented out later
	private void describeMotionEventsforDebug(View v, MotionEvent event){
		//0=ACTION_DOWN, 1=ACTION_UP, 2=ACTION_MOVE
		//Size=how big the touch is or area
		Log.v(TAG, "--- Describe MotionEvents for Visibilty! ---");
		
		StringBuilder result = new StringBuilder(20);
		int numOfPointers = event.getPointerCount(); //how many fingers on screen
		int pointerIndex = 0; //index always 0
		
		while(pointerIndex < numOfPointers){
			result.append("\n").append("------MultiTouch-------").append("\n");
			result.append("Number of pointers:  ").append(event.getPointerCount()).append("\n");
			//pointer index
			result.append("PointerIndex(My Defined Loop):  ").append(pointerIndex).append("\n");
			result.append("PointerIndex from getAction():  ").append(event.getActionIndex()).append("\n");
			result.append("PointerId:  ").append(event.getPointerId(pointerIndex)).append("\n");
			result.append("Coordinates: X: ").append(event.getX(pointerIndex)).append("  Y: ").append(event.getY(pointerIndex)).append("\n");
			result.append("Event: ").append(event.getAction()).append("\n");
			result.append("Size:  ").append(event.getSize());
			Log.v(TAG, result.toString());
		}
		
	}

	public boolean isTouched(int pointerId){ //pointerId sustained through touchevents if a finger remains
		synchronized(this){
			if(pointerId < 0 || pointerId >=20){ //no errors, android only handles 10 events
				return false;
			}
			else{
				return _isTouched[pointerId];
			}
		}
	}
	
	public int getTouchX(int pointerId){
		synchronized(this){
			if(pointerId < 0 || pointerId >=20){ 
				return 0;
			}
			else{
				return _touchX[pointerId];
			}
		}
	}
	
	public int getTouchY(int pointerId){
		synchronized(this){
			if(pointerId < 0 || pointerId >=20){ 
				return 0;
			}
			else{
				return _touchY[pointerId];
			}
		}
	}
	
	public ArrayList<TouchEvent> getTouchEvents(){
		synchronized(this){
			int length = _touchEventsBufferTwo.size();
			for(int i=0; i<length; i++){
				_touchEventPool.returnToPool(_touchEventsBufferTwo.get(i));
			}
				_touchEventsBufferTwo.clear();
				_touchEventsBufferTwo.addAll(_touchEventsBufferOne);
				_touchEventsBufferOne.clear();
			return _touchEventsBufferTwo;
		}
	}
}
